"""
View names for use in urlpatterns and reverse lookup.
"""

"""API endpoint which receives new webmentions."""
webmention_api_incoming = "incoming_webmention_api_view"

"""API endpoint for retrieving existing webmentions for some target URL or object."""
webmention_api_get_for_object = "get_webmentions_for_object_api_view"
